<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Version details
 *
 * @package    block_iclicker
 * @copyright  2012 i>clicker (R) <http://www.iclicker.com/dnn/> onwards Saji Nair (http://sajinair.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

/**
 * Handles rendering the form for creating new pages and the submission of the form as well
 * NOTE: table is named iclicker
 */

require_once('../../config.php');
global $CFG, $USER, $COURSE, $OUTPUT, $PAGE;
require_once('iclicker_service.php');
require_once('controller.php');

$site = get_site();
require_login($site);

// Activate the controller.
$cntlr = new iclicker_controller();
$cntlr->process_instructor_sso();
foreach ($cntlr->results as $key => $val) {
    $$key = $val;
}

// Begin rendering.
$title = strip_tags($site->fullname).':'.iclicker_service::msg('app.iclicker').':'.iclicker_service::msg('inst.sso.title');
$PAGE->set_title($title);
$PAGE->set_heading( iclicker_service::msg('app.iclicker').' '.iclicker_service::msg('inst.sso.title') );
$PAGE->navbar->add(iclicker_service::msg('inst.sso.title'));
$PAGE->set_focuscontrol('');
$PAGE->set_cacheable(false);
$PAGE->requires->css(iclicker_service::BLOCK_PATH.'/css/iclicker.css');
$PAGE->set_url(iclicker_service::BLOCK_PATH.'/instructor_sso.php');
echo $OUTPUT->header();
?>
<div class="iclicker">

    <?php
    // Show messages if there are any to show.
    require('user_messages.php');
    ?>

    <div class="main_content">
        <?php if ($ssoenabled) { ?>
        <div class="inst_sso_instructions">
            <?php echo iclicker_service::msg('inst.sso.instructions') ?>
        </div>
        <div class="inst_sso_controls">
            <span class="sso_control_message"><?php echo iclicker_service::msg('inst.sso.key.message') ?>: </span>
            <span class="sso_control_key"><?php echo $ssouserkey ?></span>
            <form class="sso_control_form" method="post" style="display:inline;">
                <input type="submit" class="generate_button" name="generateKey"
                       value="<?php echo iclicker_service::msg('inst.sso.generate.key');?>" />
            </form>
        </div>
        <?php } else { ?>
        <div class="error"><?php echo iclicker_service::msg('inst.sso.disabled') ?></div>
        <?php } ?>
    </div>

    <div class="nav_links">
        <?php
        $reglink = '<a class="nav_link" href="'.iclicker_service::block_url('registration.php').'">';
        $reglink .= iclicker_service::msg('reg.title').'</a>';
        $navlinks = $reglink.PHP_EOL;
        // The other links.
        $navlinks .= ' | <a class="nav_link" href="'.iclicker_service::block_url('instructor.php').'">';
        $navlinks .= iclicker_service::msg('inst.title').'</a>'.PHP_EOL;
        if (iclicker_service::$blockiclickerssoenabled) {
            $navlinks .= ' | <a class="nav_link current_nav_link" href="'.iclicker_service::block_url('instructor_sso.php').'">';
            $navlinks .= iclicker_service::msg('inst.sso.title').'</a>'.PHP_EOL;
        }
        if (iclicker_service::is_admin()) {
            $navlinks .= ' | <a class="nav_link" href="'.iclicker_service::block_url('admin.php').'">';
            $navlinks .= iclicker_service::msg('admin.title').'</a>'.PHP_EOL;
        }
        echo $navlinks;
        ?>
    </div>

    <div class="iclicker_version">
        Version <?php echo iclicker_service::VERSION ?> (<?php echo iclicker_service::BLOCK_VERSION ?>)
    </div>

</div>

<?php
    echo $OUTPUT->footer();
