<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Version details
 *
 * @package    block_iclicker
 * @copyright  2012 i>clicker (R) <http://www.iclicker.com/dnn/> onwards Saji Nair (http://sajinair.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
/* $Id: test_iclicker_service.php 161 2012-08-16 14:19:57Z azeckoski@gmail.com $ */

if (!defined('MOODLE_INTERNAL')) {
    die('Direct access to this script is forbidden.'); // It must be included from a Moodle page
}

require_once(dirname(__FILE__) . '/../../../config.php');
global $CFG, $USER, $COURSE;
// link in external libraries
// @noinspection PhpIncludeInspection */
require_once($CFG->dirroot . '/blocks/iclicker/iclicker_service.php');

/**
 * Unit tests for the iclicker services
 *
 * http://docs.moodle.org/dev/PHPUnit_integration
 *
 * resetAfterTest(bool)
 * true means reset automatically after test, false means keep changes to next test method, default null means detect changes
 * resetAllData()
 * reset global state in the middle of a test
 * setAdminUser()
 * set current $USER as admin
 * setGuestUser()
 * set current $USER as guest
 * setUser()
 * set current $USER to a specific user - use getDataGenerator() to create one
 * getDataGenerator()
 * returns data generator instance - use if you need to add new courses, users, etc.
 * preventResetByRollback()
 * terminates active transactions, useful only when test contains own database transaction handling
 * createXXXDataSet()
 * creates in memory structure of database table contents, used in loadDataSet() (eg: createXMLDataSet(), createCsvDataSet(), createFlatXMLDataSet())
 * loadDataSet()
 * bulk loading of table contents
 * getDebuggingMessages()
 * Return debugging messages from the current test. (Moodle 2.4 and upwards)
 * resetDebugging()
 * Clear all previous debugging messages in current test. (Moodle 2.4 and upwards)
 * assertDebuggingCalled()
 * Assert that exactly debugging was just called once. (Moodle 2.4 and upwards)
 * assertDebuggingNotCalled()
 * Assert no debugging happened. (Moodle 2.4 and upwards)
 * redirectMessages()
 * Captures ongoing messages for later testing (Moodle 2.4 and upwards)
 *
 * NOTE: it is not possible to modify database structure such as create new table or drop columns from advanced_testcase.
 *
 * @group block_iclicker
 */
class iclicker_services_test extends advanced_testcase {

    public $courseid = 1;
    public $clickerid = '99996666';

    public $studentid1 = 1;
    public $studentid2 = 2;
    public $instructorid = 3;
    public $instructorrole = 3;

    public $catname = 'az_category';
    public $itemname = 'az_gradeitem';
    public $gradescore = 91;

    public function setUp() {
        // setup the test data (users and course)
        $student1 = $this->getDataGenerator()->create_user(array('email' => 'user1@iclicker.com', 'username' => 'iuser1'));
        $this->studentid1 = $student1->id;
        $student2 = $this->getDataGenerator()->create_user(array('email' => 'user2@iclicker.com', 'username' => 'iuser2'));
        $this->studentid2 = $student2->id;
        $instructor = $this->getDataGenerator()->create_user(array('email' => 'inst1@iclicker.com', 'username' => 'iinst1'));
        $this->instructorid = $instructor->id;
        $category1 = $this->getDataGenerator()->create_category();
        $courseary = array('shortname' => 'ic_1', 'fullname' => 'iclicker course', 'summary' => 'iclicker course desc', 'category' => $category1->id);
        $course1 = $this->getDataGenerator()->create_course($courseary);
        $this->courseid = $course1->id;
        $this->getDataGenerator()->enrol_user($this->studentid1, $this->courseid);
        $this->getDataGenerator()->enrol_user($this->studentid1, $this->courseid);
        $this->getDataGenerator()->enrol_user($this->instructorid, $this->courseid, $this->instructorrole);

        iclicker_service::$testmode = true;
    }

    public function tearDown() {
        iclicker_service::$testmode = false;
    }

    public function test_arrays_subtract() {
        $this->resetAfterTest(true); // reset all changes automatically after this test

        // array_diff and array_diff_key are the same as a subtract when used with 2 arrays -- array_diff(A1, A2) => A1 - A2
        $a1 = array(1, 2, 3, 4, 5);
        $a2 = array(3, 4, 5, 6, 7);

        $result = array_values(array_diff($a1, $a2));
        $this->assertEquals(2, count($result));
        $this->assertTrue(in_array(1, $result));
        $this->assertTrue(in_array(2, $result));
        $result = array_values(array_diff($a2, $a1));
        $this->assertEquals(2, count($result));
        $this->assertTrue(in_array(6, $result));
        $this->assertTrue(in_array(7, $result));

        $result = array_values(array_intersect($a1, $a2));
        $this->assertEquals(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));
        $result = array_values(array_intersect($a2, $a1));
        $this->assertEquals(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));

        $a1 = array('A' => 1, 'B' => 2, 'C' => 3, 'D' => 4, 'E' => 5);
        $a2 = array('C' => 3, 'D' => 4, 'E' => 5, 'F' => 6, 'G' => 7);

        $result = array_values(array_diff_key($a1, $a2));
        $this->assertEquals(2, count($result));
        $this->assertTrue(in_array(1, $result));
        $this->assertTrue(in_array(2, $result));
        $result = array_values(array_diff_key($a2, $a1));
        $this->assertEquals(2, count($result));
        $this->assertTrue(in_array(6, $result));
        $this->assertTrue(in_array(7, $result));

        $result = array_values(array_intersect_key($a1, $a2));
        $this->assertEquals(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));
        $result = array_values(array_intersect_key($a2, $a1));
        $this->assertEquals(3, count($result));
        $this->assertTrue(in_array(3, $result));
        $this->assertTrue(in_array(4, $result));
        $this->assertTrue(in_array(5, $result));
    }

    public function test_assert() {
        $this->resetAfterTest(true); // reset all changes automatically after this test

        $this->assertEquals("AZ", "AZ");
        $this->assertEquals(iclicker_service::$testmode, true);
    }

    public function test_require_user() {
        $this->resetAfterTest(true); // reset all changes automatically after this test
        $this->setUser($this->studentid1);

        $userid = iclicker_service::require_user();
        $this->assertTrue(is_numeric($userid));
        $this->assertEquals($this->studentid1, $userid);
    }

    public function test_get_users() {
        global $DB;
        $this->resetAfterTest(true); // reset all changes automatically after this test
        $this->setUser($this->studentid1);

        $userid = iclicker_service::require_user();
        $this->assertTrue(is_numeric($userid));
        $this->assertEquals($this->studentid1, $userid);
        $results = iclicker_service::get_users(array($userid));
        $this->assertTrue(!empty($results));
        $this->assertTrue(count($results) == 1);
        $this->assertEquals($results[$userid]->id, $userid);
    }

    public function test_validate_clickerid() {
        $this->resetAfterTest(true); // reset all changes automatically after this test

        $clickerid = null;
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEquals(ClickerIdInvalidException::F_EMPTY, $e->type);
        }

        $clickerid = "XXX";
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEquals(ClickerIdInvalidException::F_CHARS, $e->type);
        }

        $clickerid = "00000000000";
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEquals(ClickerIdInvalidException::F_LENGTH, $e->type);
        }

        $clickerid = iclicker_service::CLICKERID_SAMPLE;
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEquals(ClickerIdInvalidException::F_SAMPLE, $e->type);
        }

        $clickerid = "ABCD0123";
        try {
            iclicker_service::validate_clicker_id($clickerid);
            $this->fail("should have died");
        } catch (ClickerIdInvalidException $e) {
            $this->assertEquals(ClickerIdInvalidException::F_CHECKSUM, $e->type);
        }

        $clickerid = "112233";
        $result = iclicker_service::validate_clicker_id($clickerid);
        $this->assertEquals($result, "00112233");

        $clickerid = "11111111";
        $result = iclicker_service::validate_clicker_id($clickerid);
        $this->assertEquals($result, $clickerid);
    }

    public function test_alternate_clickerid() {
        $this->resetAfterTest(true); // reset all changes automatically after this test

        $result = null;

        $result = iclicker_service::translate_clicker_id(null);
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("");
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id(iclicker_service::CLICKERID_SAMPLE);
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("11111111");
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("22222222");
        $this->assertEquals("02222202", $result);

        $result = iclicker_service::translate_clicker_id("33333333");
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("44444444");
        $this->assertEquals("04444404", $result);

        $result = iclicker_service::translate_clicker_id("55555555");
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("66666666");
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("77777777");
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("88888888");
        $this->assertEquals("08888808", $result);

        $result = iclicker_service::translate_clicker_id("99999999");
        $this->assertEquals(null, $result);

        $result = iclicker_service::translate_clicker_id("40404040");
        $this->assertEquals("00404000", $result);
    }

    public function test_make_clickerid_dates() {
        $this->resetAfterTest(true); // reset all changes automatically after this test
        $this->setUser($this->studentid1);

        $userid = iclicker_service::require_user();
        $reg1 = new stdClass;
        $reg1->clickerid = '11111111';
        $reg1->ownerid = 'aaronz';
        $reg1->timecreated = time() - 1000;
        $reg2 = new stdClass;
        $reg2->clickerid = '22222222';
        $reg2->ownerid = 'beckyz';
        $reg2->timecreated = time() - 100;
        $result = null;

        $clickers = array($reg1);
        $result = iclicker_service::make_clicker_ids_and_dates($clickers);
        $this->assertNotNull($result);
        $this->assertEquals("11111111", $result['clickerid']);

        $clickers = array($reg2);
        $result = iclicker_service::make_clicker_ids_and_dates($clickers);
        $this->assertNotNull($result);
        $this->assertEquals("22222222,02222202", $result['clickerid']);

        $clickers[] = $reg1;
        $result = iclicker_service::make_clicker_ids_and_dates($clickers);
        $this->assertNotNull($result);
        $this->assertEquals("22222222,02222202,11111111", $result['clickerid']);
    }

    public function test_alphanumeric_gen() {
        $this->resetAfterTest(true); // reset all changes automatically after this test

        $rand = iclicker_service::random_alpha_numeric(10);
        $this->assertNotNull($rand);
        $this->assertTrue(strlen($rand) == 10);

        $rand = iclicker_service::random_alpha_numeric(12);
        $this->assertNotNull($rand);
        $this->assertTrue(strlen($rand) == 12);
    }

    /**
     * Sample key:
     * abcdef1234566890
     * Sample timestamp:
     * 1332470760
     * Encoded key:
     * cc80462bfc0da7e614237d7cab4b7971b0e71e9f|1332470760
     */
    public function test_sso_key_encoding() {
        $this->resetAfterTest(true); // reset all changes automatically after this test

        $key = "abcdef1234566890";
        iclicker_service::set_shared_key($key);

        // test expired timestamp
        $encodedkey = "cc80462bfc0da7e614237d7cab4b7971b0e71e9f|1332470760";
        try {
            iclicker_service::verify_key($encodedkey);
            $this->fail("should have died");
        } catch (ClickerSecurityException $e) {
            $this->assertNotNull($e->getMessage());
        }

        // test invalid format
        try {
            iclicker_service::verify_key("xxxxxxxxxxxxx");
            $this->fail("should have died");
        } catch (InvalidArgumentException $e) {
            $this->assertNotNull($e->getMessage());
        }
        try {
            iclicker_service::verify_key("xxxxxxxxxxxxx|");
            $this->fail("should have died");
        } catch (InvalidArgumentException $e) {
            $this->assertNotNull($e->getMessage());
        }
        try {
            iclicker_service::verify_key("xxxxxxxx|12344ffff");
            $this->fail("should have died");
        } catch (InvalidArgumentException $e) {
            $this->assertNotNull($e->getMessage());
        }

        // test valid encoded key
        $timestamp = time();
        $encodedkey = sha1($key . ":" . $timestamp) . '|' . $timestamp;
        $result = iclicker_service::verify_key($encodedkey);
        $this->assertTrue($result);
        // echo "<div><b>SSO key:</b> key=$key, ts=$timestamp <br/> encoded=<input type='text' size='".(strlen($encodedkey)+2)."' value='$encodedkey'/></div>".PHP_EOL;
    }

    public function test_user_keys() {
        global $DB;
        $this->resetAfterTest(true); // reset all changes automatically after this test
        $this->setUser($this->instructorid);

        $userid = iclicker_service::require_user();
        $this->assertNotNull($userid);

        // saving key for user should work
        $userkey = iclicker_service::make_user_key($userid, true);
        $this->assertNotNull($userkey);

        $keyscount = $DB->count_records(iclicker_service::USER_KEY_TABLENAME, array('userid' => $userid));
        $this->assertEquals(1, $keyscount);

        // getting key for user (from make)
        $userkey1 = iclicker_service::make_user_key($userid, false);
        $this->assertNotNull($userkey1);
        $this->assertEquals($userkey, $userkey1);

        // getting key for user (from get)
        $userkey1 = iclicker_service::get_user_key($userid);
        $this->assertNotNull($userkey1);
        $this->assertEquals($userkey, $userkey1);

        // update the key should work
        $userkey2 = iclicker_service::make_user_key($userid, true);
        $this->assertNotNull($userkey2);
        $this->assertNotEquals($userkey, $userkey2);

        // trying to save another key for this user should fail
        $dupeuserkey = new stdClass();
        $dupeuserkey->userid = $userid;
        $dupeuserkey->userkey = 'x12xx3x3x43x44';
        try {
            $DB->insert_record(iclicker_service::USER_KEY_TABLENAME, $dupeuserkey);
            $this->fail("Should have thrown an exception");
        } catch (dml_exception $e) {
            $this->assertNotNull($e->getMessage());
        }

        // delete the key should work
        $DB->delete_records(iclicker_service::USER_KEY_TABLENAME, array('userid' => $userid));
        $userkey3 = iclicker_service::get_user_key($userid);
        $this->assertNull($userkey3);

        // adding key2 should work since key1 is gone
        $userkey4 = iclicker_service::make_user_key($userid, true);
        $this->assertNotNull($userkey4);

        // cleanup
        $DB->delete_records(iclicker_service::USER_KEY_TABLENAME, array('userid' => $userid));
    }

    public function test_registrations() {
        global $DB;
        $this->resetAfterTest(true); // reset all changes automatically after this test
        $this->setUser($this->instructorid);

        $reg = null;
        $userid = iclicker_service::require_user();

        // try get registration
        $reg = iclicker_service::get_registration_by_clicker_id($this->clickerid);
        $this->assertFalse($reg);

        // create registration
        $reg = iclicker_service::create_clicker_registration($this->clickerid, $userid);
        $this->assertTrue(!empty($reg));
        $this->assertEquals($this->clickerid, $reg->clickerid);
        $regid = $reg->id;
        $this->assertTrue(!empty($regid));
        $this->assertEquals(0, $reg->fromnational);

        // get registration
        $reg1 = iclicker_service::get_registration_by_clicker_id($this->clickerid, $userid);
        $this->assertTrue(!empty($reg1));
        $this->assertEquals($this->clickerid, $reg1->clickerid);
        $this->assertEquals($regid, $reg1->id);

        $reg2 = iclicker_service::get_registration_by_id($regid);
        $this->assertTrue(!empty($reg2));
        $this->assertEquals($this->clickerid, $reg2->clickerid);
        $this->assertEquals($regid, $reg2->id);

        // save registration
        $reg->fromnational = 1;
        $saveid = iclicker_service::save_registration($reg);
        $this->assertTrue(!empty($saveid));
        $this->assertEquals($regid, $saveid);

        // check it changed
        $reg3 = iclicker_service::get_registration_by_id($regid);
        $this->assertTrue(!empty($reg3));
        $this->assertEquals($regid, $reg3->id);
        $this->assertEquals(1, $reg3->fromnational);
        // too fast $this->assertNotEquals($reg->timemodified, $reg3->timemodified);

        // make registration inactive
        $this->assertEquals(1, $reg->activated);
        $reg4 = iclicker_service::set_registration_active($regid, false);
        $this->assertTrue(!empty($reg4));
        $this->assertEquals(0, $reg4->activated);
        // check it changed
        $reg5 = iclicker_service::get_registration_by_id($regid);
        $this->assertTrue(!empty($reg5));
        $this->assertEquals($regid, $reg5->id);
        $this->assertEquals($reg4->id, $reg5->id);
        $this->assertEquals(0, $reg5->activated);

        // get all registration
        $results = iclicker_service::get_registrations_by_user($userid);
        $this->assertTrue(!empty($results));
        $this->assertEquals(1, count($results));

        $results = iclicker_service::get_registrations_by_user($userid, true);
        $this->assertNotNull($results);
        $this->assertFalse(!empty($results));
        $this->assertEquals(0, count($results));

        $this->setAdminUser(); // MUST be admin
        $results = iclicker_service::get_all_registrations();
        $this->assertTrue(!empty($results));
        $this->assertTrue(count($results) >= 1);

        // remove registration
        $result = iclicker_service::remove_registration($regid);
        $this->assertTrue(!empty($results));

        // try get registration
        $reg = iclicker_service::get_registration_by_id($regid);
        $this->assertFalse($reg);
    }

    public function test_encode_decode() {
        $this->resetAfterTest(true); // reset all changes automatically after this test

        $xml = <<<XML
<Register>
  <S DisplayName="DisplayName-azeckoski-123456" FirstName="First" LastName="Lastazeckoski-123456"
    StudentID="101" Email="azeckoski-123456@email.com" URL="http://sakaiproject.org" ClickerID="11111111"></S>
</Register>
XML;
        $result = iclicker_service::decode_registration($xml);
        $this->assertNotNull($result);
        $this->assertNotNull($result->clickerid);
        $this->assertNotNull($result->ownerid);
        $this->assertEquals($result->clickerid, '11111111');
        $this->assertEquals($result->ownerid, 101);
        // $this->assertEquals($result->user_username, 'student01'); // DISABLED - username might be different for this user id

        $xml = <<<XML
<coursegradebook courseid="BFW61">
  <user id="101" usertype="S">
    <lineitem name="05/05/2009" pointspossible="100" type="iclicker polling scores" score="100"/>
    <lineitem name="06/06/2009" pointspossible="50" type="iclicker polling scores" score="50"/>
  </user>
  <user id="102" usertype="S">
    <lineitem name="06/06/2009" pointspossible="50" type="iclicker polling scores" score="30"/>
    <lineitem name="07/07/2009" pointspossible="100" type="iclicker polling scores" score="80"/>
  </user>
</coursegradebook>
XML;
        $result = iclicker_service::decode_gradebook($xml);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertEquals($result->course_id, 'BFW61');
        $this->assertNotNull($result->students);
        $this->assertNotNull($result->items);
        $this->assertEquals(count($result->students), 2);
        $this->assertEquals(count($result->items), 3);
        $this->assertNotNull($result->items['05/05/2009']);
        $this->assertNotNull($result->items['06/06/2009']);
        $this->assertNotNull($result->items['07/07/2009']);

        $xml = <<<XML
<StudentRoster>
    <S StudentID="student01" FirstName="student01" LastName="student01" URL="https://www.iclicker.com/" CourseName="">
        <Registration ClickerId="11111111" WhenAdded="2009-01-27" Enabled="True" />
        <Registration ClickerId="22222222" WhenAdded="2009-01-27" Enabled="True" />
    </S>
</StudentRoster>
XML;
        $result = iclicker_service::decode_ws_xml($xml);
        $this->assertNotNull($result);
        $this->assertTrue(is_array($result));
        $this->assertEquals(count($result), 2);
        $this->assertNotNull($result[0]);
        $this->assertNotNull($result[0]->clickerid);
        $this->assertNotNull($result[0]->ownerid);
        $this->assertNotNull($result[0]->timecreated);
        $this->assertNotNull($result[0]->activated);
        $this->assertEquals($result[0]->clickerid, '11111111');
        $this->assertEquals($result[0]->user_username, 'student01');
        /*
        Fails in windows for some reason - not sure why
        Fail: blocks/iclicker/simpletest/test_iclicker_service.php / ► iclicker_services_test / ► test_encode_decode / ►
        Equal expectation fails because [Integer: 1233032400] differs from [Integer: 1233014400] by 18000
        */
        // $this->assertEquals($result[0]->timecreated, 1233014400);
        $this->assertEquals($result[0]->activated, true);

        // no good way to test this right now
        // $result = iclicker_service::encode_courses($instructorid);
        // $result = iclicker_service::encode_enrollments($courseid);
        // $result = iclicker_service::encode_gradebook_results($courseid, $result_items);

        $clickerregistration = new stdClass();
        $clickerregistration->ownerid = 101;
        $clickerregistration->clickerid = '12345678';
        $clickerregistration->activated = true;
        $result = iclicker_service::encode_registration($clickerregistration);
        $this->assertNotNull($result);
        // $this->assertTrue(stripos($result, 'student01') > 0); // DISABLED - username might be different for this user id
        $this->assertTrue(stripos($result, '12345678') > 0);
        $this->assertTrue(stripos($result, 'True') > 0);

        $registrations = array($clickerregistration);
        $result = iclicker_service::encode_registration_result($registrations, true, 'hello');
        $this->assertNotNull($result);
        $this->assertTrue(stripos($result, 'True') > 0);
        $this->assertTrue(stripos($result, 'hello') > 0);
        $result = iclicker_service::encode_registration_result($registrations, false, 'failed');
        $this->assertNotNull($result);
        $this->assertTrue(stripos($result, 'False') > 0);
        $this->assertTrue(stripos($result, 'failed') > 0);

    }

    public function test_save_grades() {
        global $DB;
        $this->resetAfterTest(true); // reset all changes automatically after this test
        $this->setUser($this->instructorid);

        $testitemname1 = 'testing-iclicker-item1';
        $testitemname2 = 'testing-iclicker-item2';
        $testitemname3 = 'testing-iclicker-item3';

        $gradebook = new stdClass();

        // saving a gradebook with no course_id not allowed
        try {
            $result = iclicker_service::save_gradebook($gradebook);
            $this->fail("should have died");
        } catch (Exception $e) {
            $this->assertNotNull($e);
        }

        $gradebook->course_id = $this->courseid;
        $gradebook->items = array();

        // saving an empty gradebook not allowed
        try {
            $result = iclicker_service::save_gradebook($gradebook);
            $this->fail("should have died");
        } catch (Exception $e) {
            $this->assertNotNull($e);
        }

        // saving one with one valid item
        $score = new stdClass();
        $score->userid = 101;
        $score->score = 75.0;

        $gradeitem = new stdClass();
        $gradeitem->name = $testitemname1;
        $gradeitem->points_possible = 90;
        $gradeitem->type = iclicker_service::GRADE_CATEGORY_NAME;
        $gradeitem->scores = array();

        $gradeitem->scores[] = $score;
        $gradebook->items[] = $gradeitem;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->course);
        $this->assertNotNull($result->default_category_id);
        $this->assertNotNull($result->items);
        $this->assertEquals($result->course_id, $this->courseid);
        $this->assertEquals(count($result->items), 1);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEquals(count($result->items[0]->scores), 1);
        $this->assertFalse(isset($result->items[0]->errors));
        $this->assertEquals($result->items[0]->grademax, 90);
        $this->assertEquals($result->items[0]->iteminfo, iclicker_service::GRADE_CATEGORY_NAME);
        $this->assertNotNull($result->items[0]->categoryid);
        $this->assertEquals($result->items[0]->courseid, $result->course_id);
        $this->assertEquals($result->items[0]->itemname, $testitemname1);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertFalse(isset($result->items[0]->scores[0]->error));

        // saving one with multiple items, some invalid
        $gradeitem->type = 'stuff'; // update category
        $score->score = 50; // SCORE_UPDATE_ERRORS

        $score1 = new stdClass();
        $score1->userid = 'xxxxxx'; // USER_DOES_NOT_EXIST_ERROR
        $score1->score = 80;
        $gradeitem->scores[] = $score1;

        $score2 = new stdClass();
        $score2->userid = '102';
        $score2->score = 101; // POINTS_POSSIBLE_UPDATE_ERRORS
        $gradeitem->scores[] = $score2;

        $score3 = new stdClass();
        $score3->userid = '103';
        $score3->score = 'XX'; // GENERAL_ERRORS
        $gradeitem->scores[] = $score3;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->course);
        // $this->assertNotNull($result->default_category_id);
        $this->assertNotNull($result->items);
        $this->assertEquals($result->course_id, $this->courseid);
        $this->assertEquals(count($result->items), 1);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertEquals($result->items[0]->iteminfo, 'stuff');
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEquals(count($result->items[0]->scores), 4);
        $this->assertTrue(isset($result->items[0]->errors));
        $this->assertEquals(count($result->items[0]->errors), 4);
        $this->assertEquals($result->items[0]->grademax, 90);
        $this->assertNotNull($result->items[0]->categoryid);
        $this->assertEquals($result->items[0]->courseid, $result->course_id);
        $this->assertEquals($result->items[0]->itemname, $testitemname1);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertTrue(isset($result->items[0]->scores[0]->error));
        $this->assertEquals($result->items[0]->scores[0]->error, iclicker_service::SCORE_UPDATE_ERRORS);
        $this->assertNotNull($result->items[0]->scores[1]);
        $this->assertTrue(isset($result->items[0]->scores[1]->error));
        $this->assertEquals($result->items[0]->scores[1]->error, iclicker_service::USER_DOES_NOT_EXIST_ERROR);
        $this->assertNotNull($result->items[0]->scores[2]);
        $this->assertTrue(isset($result->items[0]->scores[2]->error));
        $this->assertEquals($result->items[0]->scores[2]->error, iclicker_service::POINTS_POSSIBLE_UPDATE_ERRORS);
        $this->assertNotNull($result->items[0]->scores[3]);
        $this->assertTrue(isset($result->items[0]->scores[3]->error));
        $this->assertEquals($result->items[0]->scores[3]->error, 'SCORE_INVALID'); // iclicker_service::USER_DOES_NOT_EXIST_ERROR);

        $xml = iclicker_service::encode_gradebook_results($result);
        $this->assertNotNull($xml);
        $this->assertTrue(stripos($xml, '<user ') > 0);
        $this->assertTrue(stripos($xml, '<lineitem ') > 0);
        $this->assertTrue(stripos($xml, '<error ') > 0);
        $this->assertTrue(stripos($xml, iclicker_service::SCORE_UPDATE_ERRORS) > 0);
        $this->assertTrue(stripos($xml, iclicker_service::USER_DOES_NOT_EXIST_ERROR) > 0);
        $this->assertTrue(stripos($xml, iclicker_service::POINTS_POSSIBLE_UPDATE_ERRORS) > 0);
        $this->assertTrue(stripos($xml, iclicker_service::GENERAL_ERRORS) > 0);
        // echo "<xmp>$xml</xmp>";

        // Save 1 update and 2 new grades
        $score->score = 85;
        $score2->score = 50;
        $score3->score = 0;
        $gradeitem->scores = array();
        $gradeitem->scores[] = $score;
        $gradeitem->scores[] = $score2;
        $gradeitem->scores[] = $score3;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->items);
        $this->assertEquals($result->course_id, $this->courseid);
        $this->assertEquals(count($result->items), 1);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEquals(count($result->items[0]->scores), 3);
        $this->assertFalse(isset($result->items[0]->errors));
        $this->assertEquals($result->items[0]->grademax, 90);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertEquals($result->items[0]->scores[0]->rawgrade, 85);
        $this->assertNotNull($result->items[0]->scores[1]);
        $this->assertEquals($result->items[0]->scores[1]->rawgrade, 50);
        $this->assertNotNull($result->items[0]->scores[2]);
        $this->assertEquals($result->items[0]->scores[2]->rawgrade, 0);
        /*
        echo "<pre>";
        var_export($result->items[0]);
        echo "</pre>";
        */
        $xml = iclicker_service::encode_gradebook_results($result);
        $this->assertNull($xml); // no errors

        // test saving multiple items at once
        $gradebook->course_id = $this->courseid;
        $gradebook->items = array();

        $gradeitem1 = new stdClass();
        $gradeitem1->name = $testitemname2;
        $gradeitem1->points_possible = 100;
        $gradeitem1->type = null; // default
        $gradeitem1->scores = array();
        $gradebook->items[] = $gradeitem1;

        $gradeitem2 = new stdClass();
        $gradeitem2->name = $testitemname3;
        $gradeitem2->points_possible = 50;
        $gradeitem2->type = 'stuff';
        $gradeitem2->scores = array();
        $gradebook->items[] = $gradeitem2;

        $score = new stdClass();
        $score->userid = 101;
        $score->score = 80.0;
        $gradeitem1->scores[] = $score;

        $score = new stdClass();
        $score->userid = 102;
        $score->score = 90.0;
        $gradeitem1->scores[] = $score;

        $score = new stdClass();
        $score->userid = 102;
        $score->score = 45.0;
        $gradeitem2->scores[] = $score;

        $score = new stdClass();
        $score->userid = 103;
        $score->score = 40.0;
        $gradeitem2->scores[] = $score;

        $result = iclicker_service::save_gradebook($gradebook);
        $this->assertNotNull($result);
        $this->assertNotNull($result->course_id);
        $this->assertNotNull($result->items);
        $this->assertNotNull($result->default_category_id);
        $this->assertEquals($result->course_id, $this->courseid);
        $this->assertEquals(count($result->items), 2);
        $this->assertNotNull($result->items[0]);
        $this->assertNotNull($result->items[0]->id);
        $this->assertNotNull($result->items[0]->scores);
        $this->assertEquals(count($result->items[0]->scores), 2);
        $this->assertFalse(isset($result->items[0]->errors));
        $this->assertEquals($result->items[0]->grademax, 100);
        $this->assertNotNull($result->items[0]->scores[0]);
        $this->assertEquals($result->items[0]->scores[0]->rawgrade, 80);
        $this->assertNotNull($result->items[0]->scores[1]);
        $this->assertEquals($result->items[0]->scores[1]->rawgrade, 90);
        $this->assertNotNull($result->items[1]);
        $this->assertNotNull($result->items[1]->id);
        $this->assertNotNull($result->items[1]->scores);
        $this->assertEquals(count($result->items[1]->scores), 2);
        $this->assertFalse(isset($result->items[1]->errors));
        $this->assertEquals($result->items[1]->grademax, 50);
        $this->assertNotNull($result->items[1]->scores[0]);
        $this->assertEquals($result->items[1]->scores[0]->rawgrade, 45);
        $this->assertNotNull($result->items[1]->scores[1]);
        $this->assertEquals($result->items[1]->scores[1]->rawgrade, 40);

        $xml = iclicker_service::encode_gradebook_results($result);
        $this->assertNull($xml);

    }

}
